using System.Net;
using System.Reflection;
using System.Security.Cryptography;
using System.Text;
using Autofac;
using Autofac.Core.Activators.Reflection;
using FitConnect.Encryption;
using FitConnect.Interfaces.Subscriber;
using FitConnect.Models;
using FitConnect.Models.v1.Api;
using FitConnect.Services.Models.v1.Submission;
using IdentityModel;
using Microsoft.AspNetCore.Http;
using Microsoft.Extensions.Logging;
using Newtonsoft.Json;
using NJsonSchema;
using NJsonSchema.Validation;
using Metadata = FitConnect.Models.Api.Metadata.Metadata;

namespace FitConnect;

/// <summary>
///     Fluent API for the FitConnect.Subscriber
/// </summary>
public class Subscriber : FitConnectClient,
    ISubscriber,
    ISubscriberWithSubmission {
    public Subscriber(FitConnectEnvironment environment,
        string clientId, string clientSecret,
        string privateKeyDecryption,
        string privateKeySigning,
        string publicKeyEncryption,
        string publicKeySignatureVerification,
        IContainer container) : base(environment, clientId, clientSecret,
        container,
        privateKeyDecryption, privateKeySigning, publicKeyEncryption,
        publicKeySignatureVerification) {
        Encryption = new FitEncryption(privateKeyDecryption, privateKeySigning, publicKeyEncryption,
            publicKeySignatureVerification,
            container.Resolve<ILogger>());
    }

    public Subscriber(FitConnectEnvironment environment,
        string clientId, string clientSecret,
        string privateKeyDecryption,
        string privateKeySigning,
        string publicKeyEncryption,
        string publicKeySignatureVerification,
        ILogger? logger = null) : base(environment, clientId, clientSecret, logger,
        privateKeyDecryption, privateKeySigning) {
        Encryption = new FitEncryption(privateKeyDecryption, privateKeySigning, publicKeyEncryption,
            publicKeySignatureVerification, logger);
    }


    /// <summary>
    ///     Receives the available submissions from the server
    /// </summary>
    /// <param name="destinationId"></param>
    /// <param name="skip"></param>
    /// <param name="take"></param>
    /// <returns></returns>
    public IEnumerable<SubmissionForPickupDto> GetAvailableSubmissions(string? destinationId = null,
        int skip = 0,
        int take = 100) {
        var submissionsResult = SubmissionService.ListSubmissions(destinationId, 0, 100).Result;

        // Creating a dictionary of destinationId to submissionIds from the REST API result
        return submissionsResult.Submissions;
    }


    /// <summary>
    ///     Receives a specific submission from the server.
    /// </summary>
    /// <param name="submissionId"></param>
    /// <param name="skipSchemaTest"></param>
    /// <returns></returns>
    public ISubscriberWithSubmission RequestSubmission(string? submissionId,
        bool skipSchemaTest = false) {
        if (submissionId == null) {
            throw new ArgumentNullException($"{nameof(submissionId)} has to be set");
        }

        var submission = (Submission)SubmissionService.GetSubmission(submissionId);
        var (metaDataString, _, metaHash) = Encryption.Decrypt(submission.EncryptedMetadata!);

        if (!skipSchemaTest) {
            var errors = VerifyMetadata(metaDataString);
            if (errors.Count > 0) {
                Logger?.LogWarning("Invalid metadata: {MetaData}", metaDataString);
                foreach (var error in errors) Logger?.LogError("Error: {Error}", error.ToString());

                throw new Exception($"Metadata validation failed: {string.Join(", ", errors)}");
            }
        }

        submission.Metadata = JsonConvert.DeserializeObject<Metadata>(metaDataString);


        if (submission.EncryptedData != null) {
            var (dataString, _, dataHash) = Encryption.Decrypt(submission.EncryptedData);
            submission.Data = dataString;

            if (submission?.Metadata?.ContentStructure.Data.Hash.Content !=
                FitEncryption.CalculateHash(dataString)) {
                Logger?.LogWarning("Data hash mismatch: {DataHash} != {CalculatedHash}",
                    submission?.Metadata?.ContentStructure.Data.Hash.Content,
                    FitEncryption.CalculateHash(dataString));
                throw new Exception("Data hash mismatch");
            }
        }

        Submission = submission;
        return this;
    }

    public Submission? Submission { get; private set; }

    /// <summary>
    ///     Reading attachments for a submission.
    /// </summary>
    /// <returns></returns>
    public IEnumerable<Attachment> GetAttachments() {
        // TODO add guard calls

        var attachments = new List<Attachment>();
        foreach (var id in Submission!.AttachmentIds) {
            var encryptedAttachment = SubmissionService.GetAttachment(Submission.Id, id);
            var (_, content, hash) = Encryption.Decrypt(encryptedAttachment);
            var attachmentMeta =
                Submission.Metadata.ContentStructure.Attachments.First(a => a.AttachmentId == id);

            attachments.Add(new Attachment(attachmentMeta, content));
        }

        Submission.Attachments = attachments;
        return attachments;
    }

    public void AcceptSubmission() {
        CompleteSubmission(Submission!, FinishSubmissionStatus.Accepted);
    }

    public void RejectSubmission(params Problems[] problems) {
        CompleteSubmission(Submission!, FinishSubmissionStatus.Rejected, problems);
    }

    public void ForwardSubmission() {
        CompleteSubmission(Submission!, FinishSubmissionStatus.Forwarded);
    }

    public void CompleteSubmission(FinishSubmissionStatus status) {
        CompleteSubmission(Submission!, status);
    }


    /// <summary>
    ///     Verify the metadata hash and content to fit the schema
    /// </summary>
    /// <param name="metadataString"></param>
    /// <returns></returns>
    public static ICollection<ValidationError> VerifyMetadata(string metadataString) {
        var schemaString = LoadContentOfResource("metadata.schema.json");
        var schema = JsonSchema.FromJsonAsync(schemaString).Result;
        return schema.Validate(metadataString);
    }

    private static string LoadContentOfResource(string resourceName) {
        var assembly = Assembly.GetExecutingAssembly();
        var fullQualifiedName = $"{assembly.GetName().Name}.{resourceName}";
        var resourceStream = assembly.GetManifestResourceStream(fullQualifiedName);
        var reader = new StreamReader(resourceStream);
        return reader.ReadToEnd();
    }


    private string GetEvent(FinishSubmissionStatus state) {
        return state switch {
            FinishSubmissionStatus.Accepted =>
                "https://schema.fitko.de/fit-connect/events/accept-submission",
            FinishSubmissionStatus.Rejected =>
                "https://schema.fitko.de/fit-connect/events/reject-submission",
            FinishSubmissionStatus.Forwarded =>
                "https://schema.fitko.de/fit-connect/events/forward-submission",
            _ => throw new ArgumentException("Invalid state")
        };
    }

    public void CompleteSubmission(SubmissionForPickupDto submission,
        FinishSubmissionStatus status, Problems[]? problems = null) {
        if (submission.SubmissionId == null || submission.CaseId == null ||
            submission.DestinationId == null)
            throw new ArgumentException("Submission does not contain all required fields");

        if (status != FinishSubmissionStatus.Rejected && problems != null)
            throw new ArgumentException("Problems can only be set for rejected submissions");

        var eventName = GetEvent(status);

        var token =
            Encryption.CreateSecurityEventToken(submission.SubmissionId, submission.CaseId,
                submission.DestinationId, eventName, problems);

        var result = CasesService.FinishSubmission(submission.CaseId, token);
        Logger?.LogInformation("Submission completed {status}", result);
    }

    public static string VerifyCallback(string callbackSecret,
        long timestamp, string body) {
        if (timestamp < DateTime.Now.AddMinutes(-5).ToEpochTime())
            throw new ArgumentException("Request is too old");

        var hmac = new HMACSHA512(Encoding.UTF8.GetBytes(callbackSecret))
            .ComputeHash(Encoding.UTF8.GetBytes($"{timestamp}.{body}"));


        return Convert.ToHexString(hmac).ToLower();
    }

    public static bool VerifyCallback(string callbackSecret, HttpRequest request) {
        if (!request.Headers.ContainsKey("callback-timestamp"))
            throw new ArgumentException("Missing callback-timestamp header");

        var timeStampString = request.Headers["callback-timestamp"].ToString();
        if (!long.TryParse(timeStampString, out var timestamp)) {
            throw new ArgumentException("Invalid callback-timestamp header");
        }


        var authentication = request.Headers["callback-authentication"];
        using var requestStream = request.Body;
        var content = new StreamReader(requestStream).ReadToEnd().Trim();

        var result = VerifyCallback(callbackSecret, timestamp, content);
        if (result != authentication)
            throw new ArgumentException("Request is not authentic");
        return true;
    }
}

public enum FinishSubmissionStatus {
    Accepted,
    Rejected,
    Forwarded
}
